#!/usr/bin/perl

# Code Borrowed from snmpnetstat template
# --------------------------------------------------
# ARGV[0] = <hostname>          required
# ARGV[1] = <snmp version>      required
# ARGV[2] = <snmp community>    required
# ARGV[3] = <snmp port>         required
# ARGV[4] = <snmp timeout>      required
# ARGV[5] = fan|ps|temp         required
# --------------------------------------------------
$in_hostname    = $ARGV[0] if defined $ARGV[0];
$in_version     = $ARGV[1] if defined $ARGV[1];
$in_community   = $ARGV[2] if defined $ARGV[2];
$in_port        = $ARGV[3] if defined $ARGV[3];
$in_timeout     = $ARGV[4] if defined $ARGV[4];
$in_type        = $ARGV[5] if defined $ARGV[5];


# usage notes
if (
        ( ! defined $in_hostname ) ||
        ( ! defined $in_version ) ||
        ( ! defined $in_community ) ||
        ( ! defined $in_port ) ||
        ( ! defined $in_timeout ) ||
        ( ! defined $in_type )
        ) {
        print   "usage:\n\n
                $0 <host> <snmp version> < snmp community> <snmp port> <snmp timeout> <query_type>\n\n";
        exit;
}

if ($in_version == 2)
{
        $in_version = "2c";
}
# end borrowed code

# Define where our SNMP utilities are.
$snmpwalk="/usr/bin/snmpwalk";
$snmpget="/usr/bin/snmpget";

# Define the OIDs to use here. 
$cduloadoid=".1.3.6.1.4.1.1718.3.2.2.1.7";
$cdupoweroid=".1.3.6.1.4.1.1718.3.2.2.1.12";

# Single OIDs to obtain thresholds. These should not vary across the towers.
$cduloadthreshoid=".1.3.6.1.4.1.1718.3.2.2.1.8.1.1";
$cdufeedcapacityoid=".1.3.6.1.4.1.1718.3.2.2.1.10.1.1";


# Some global arrays are defined here.
my @loads;
my @powers;



if ($in_type eq 'load') {

   # Grab the singleton items with snmpget. This is silly. Cacti seems limited in what SNMP queries can do,
   # Thus these values are obtained, and stored, everytime in the RRDs. It's a waste, but a hack to get this
   # extra functionality working. The 'vesq' outputs ONLY the value, nothing else.
   $cduloadthreshold = `$snmpget -t $in_timeout -v $in_version -O vesq -c $in_community $in_hostname $cduloadthreshoid`;
   $cdufeedcapacity = `$snmpget -t $in_timeout -v $in_version -O vesq -c $in_community $in_hostname $cdufeedcapacityoid`;
   chomp($cduloadthreshold);
   chomp($cdufeedcapacity);

   # Walk to get the loads. I know what order they will be in, for a two tower, 3phase unit (in this case a
   # CW series unit with a CX expansion module hanging off of it) Mileage WILL vary with this most likely.
   # New Data Input methods will have to be defined for 2phase units, and perhaps Smart units which have two "towers"
   @loads = `$snmpwalk -t $in_timeout -v $in_version -O vesq -c $in_community $in_hostname $cduloadoid`;
   chomp(@loads);

   # Dump the data so cacti can pick it up. We do NO error checking here, leave it up to cacti to treat the 
   # garbage we get back from failed snmp queries in the appropriate manner and throw it out. But it will
   # generate errors in the cacti logs. Which is a good thing?
   print "infeedLoadValue.AA:@loads[0] infeedLoadValue.AB:@loads[1] infeedLoadHighThresh:$cduloadthreshold infeedCapacity:$cdufeedcapacity";

}

elsif ($in_type eq 'power') {

   # Jump straight into a walk. We have no singleton items for the power query
   @powers = `$snmpwalk -t $in_timeout -v $in_version -O vesq -c $in_community $in_hostname $cdupoweroid`;
   chomp(@powers);

   # Same as above, just output everything, even if it's garbage. These are power distribution units we're talking
   # about. They should always behave as expected ...
   print "infeedPower.AA:@powers[0] infeedPower.AB:@powers[1]";

}

else {

   # Friendly, but useless error output :)
   print "Please Check your syntax. Something went wrong!\n";

}


__END__

=head1 NAME

ServerTech-CDU-1x2 - obtain power and load information from server technology CDUs.

=head1 VERSION

This documentation refers to ServerTech-CDU-1x2 version 1.2

=head1 APPLICATION REQUIREMENTS

 You do NOT need the MIB file for this to work. I use hardcoded OIDs that hopefully won't 
 change between firmware releases. 

=head1 USAGE

 ServerTech-CDU-1x2 <host> <snmp version> < snmp community> <snmp port> <snmp timeout> <query_type>

=head1 REQUIRED ARGUMENTS

 Everything is required, but will be provided by cacti.

=head1 MORE INFORMATION

=head2 Extra Stuff

 None at this time. I will improve documentation on subsequent releases

=head1 INCOMPATIBILITIES

 None. See Bugs.

=head1 BUGS AND LIMITATIONS

 None.
 If you experience any problems please contact me. (eric.schoeller <at> coloradoDOTedu)

=head1 AUTHOR

Eric Schoeller (eric.schoeller <at> coloradoDOTedu)

=head1 LICENCE AND COPYRIGHT

Copyright (c) 2008 Eric Schoeller (eric.schoeller <at> coloradoDOTedu).
All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License.
See L<http://www.fsf.org/licensing/licenses/gpl.html>.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
